#include <winver.h>
#include "autoconf.hh"

VS_VERSION_INFO VERSIONINFO
    FILEVERSION 0,9,8
    PRODUCTVERSION 0,9,8
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "04090000"
        BEGIN
            VALUE "ProductName", PACKAGE_STRING
            VALUE "FileDescription", "PDF to DjVu converter\0"
            VALUE "FileVersion", PACKAGE_VERSION
            VALUE "LegalCopyright", "Copyright © 2007-2016 Jakub Wilk <jwilk@jwilk.net>\0"
            VALUE "Comments", "pdf2djvu is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published by the Free Software Foundation.\0"
        END
    END
END

// vim:ts=4 sts=4 sw=4 et
